@@ -1,4 +1,4 @@
-{-# language TypeSynonymInstances, FlexibleContexts, FlexibleInstances, UndecidableInstances, OverlappingInstances, IncoherentInstances, PatternSignatures, DeriveDataTypeable, OverloadedStrings, LambdaCase, DataKinds, GADTs, QuasiQuotes #-}
+{-# language TypeSynonymInstances, FlexibleContexts, FlexibleInstances, UndecidableInstances, OverlappingInstances, IncoherentInstances, PatternSignatures, DeriveDataTypeable, OverloadedStrings, LambdaCase, DataKinds, GADTs, QuasiQuotes, TypeApplications #-}
 
 {-# OPTIONS_GHC -Werror=incomplete-patterns #-}
 
@@ -24,6 +24,7 @@ import qualified Data.Time as T
 import Control.Monad
 import Data.Typeable
 import Data.Ratio
+import Numeric.Natural
 import Data.String (fromString)
 
 tox :: CertificationProblem -> Document 
@@ -38,8 +39,11 @@ instance XmlContent CertificationProblem where
    parseContents = error "parseContents not implemented"
 
    toContents cp = rmkel "certificationProblem"
-         [ mkel "input" $ toContents ( input cp )
-         , mkel "cpfVersion" [ nospaceString $ cpfVersion cp ]
+         [ mkel "cpfVersion" [ nospaceString $ cpfVersion cp ]
+         , mkel "lookupTables"  []
+         , mkel "input" $ toContents ( input cp )
+         , mkel "property" $ rmkel "termination" []
+         , mkel "answer" $ rmkel "yes" [] -- FIXME (no?)
          , mkel "proof" $ toContents ( proof cp )
          , mkel "origin" $ toContents ( origin cp )
          ]
@@ -62,7 +66,16 @@ instance XmlContent CertificationProblemInput where
    parseContents = error "parseContents not implemented"
 
    toContents i = case i of
-      TrsInput {} -> rmkel "trsInput" $ toContents (trsinput_trs i )
+      TrsInput {} ->
+        let sys = trsinput_trs i
+            s = standard sys
+            r = relative sys
+        in  rmkel "trsInput" $
+                toContents s
+                <> if null (T.rules r) then []
+                   else rmkel "relativeRules" $
+                        rmkel "rules" $
+                        concatMap toContents $ T.rules r
       ComplexityInput {} -> rmkel "complexityInput" $ concat
           [ rmkel "trsInput" $ toContents $ trsinput_trs i
           ]
@@ -79,8 +92,8 @@ instance ( Typeable t, XmlContent t  )
    parseContents = error "parseContents not implemented"
 
    toContents u = rmkel "rule" $ concat
-      [ rmkel "lhs" ( toContents $ T.lhs u )
-      , rmkel "rhs" ( toContents $ T.rhs u )
+      [ toContents $ T.lhs u
+      , toContents $ T.rhs u
       ]
 
 instance XmlContent Proof where
@@ -109,7 +122,7 @@ instance XmlContent (TrsTerminationProof Standard) where
       RIsEmpty -> rmkel "rIsEmpty" []
       DpTrans {} -> rmkel "dpTrans" $ concat
           [ toContents $ dptrans_dps p
-          , rmkel "markedSymbols" [ nospaceString "true" ]
+          -- , rmkel "markedSymbols" [ nospaceString "true" ]
           , toContents $ dptrans_dpProof p
           ]
       StringReversal {} -> rmkel "stringReversal" $ concat
@@ -134,7 +147,7 @@ instance XmlContent (TrsTerminationProof Standard) where
           ]
       RuleRemoval {} -> rmkel "ruleRemoval" $ concat
           [ toContents $ rr_orderingConstraintProof p
-          , toContents $ trs p
+          , toContents $ trs_deleted p
           , toContents $ trsTerminationProof p
           ]
       Bounds {} -> rmkel "bounds" $ concat
@@ -149,7 +162,7 @@ instance XmlContent (TrsTerminationProof Standard) where
 instance XmlContent (TrsTerminationProof Relative) where
    parseContents = error "parseContents not implemented"
 
-   toContents p = rmkel "relativeTerminationProof" $ case p of
+   toContents p = rmkel "trsTerminationProof" $ case p of
       RIsEmpty -> rmkel "rIsEmpty" []
       SIsEmpty {} -> rmkel "sIsEmpty" $ concat
           [ toContents $ trsTerminationProof_Standard p
@@ -174,8 +187,7 @@ instance XmlContent (TrsTerminationProof Relative) where
           ]
       RuleRemoval {} -> rmkel "ruleRemoval" $ concat
           [ toContents $ rr_orderingConstraintProof p
-          , toContents $ standard $ trs p
-          , toContents $ relative $ trs p
+          , toContents $ trs_deleted p
           , toContents $ trsTerminationProof p
           ]
       EqualityRemoval {} -> rmkel "equalityRemoval" $ concat
@@ -285,10 +297,12 @@ instance XmlContent DpProof where
 instance XmlContent DepGraphComponent where
     toContents dgc = rmkel "component" $ concat $
         [ {- rmkel "dps" $ -} toContents $ dgcDps dgc
+{-        
         , rmkel "realScc" 
            --  $ toContents $ dgcRealScc dgc
            -- NO, Bool is encoded as text, not as attribute
-            $ toContents $ dgcRealScc dgc 
+            $ toContents $ dgcRealScc dgc
+-}
         ] ++ 
         [ {- rmkel "dpProof" $ -} toContents $ dgcDpProof dgc
         | dgcRealScc dgc
@@ -297,13 +311,16 @@ instance XmlContent DepGraphComponent where
 instance XmlContent OrderingConstraintProof where
   parseContents = error "parseContents not implemented"
 
-  toContents (OCPRedPair rp) = rmkel "orderingConstraintProof" 
-                             $ toContents rp
+  toContents (OCPRedPair rp) = id
+    -- $ rmkel "orderingConstraintProof" 
+    $ toContents rp
            
 instance XmlContent RedPair where
   parseContents = error "parseContents not implemented"
 
-  toContents rp = rmkel "redPair" $ case rp of
+  toContents rp = id
+          --  $ rmkel "redPair"
+          $ case rp of
     RPInterpretation i -> toContents i
     RPPathOrder      o -> toContents o
 
@@ -317,11 +334,23 @@ instance XmlContent Interpretation where
 instance XmlContent Interpretation_Type where
    parseContents = error "parseContents not implemented"
 
-   toContents t = rmkel "matrixInterpretation" $ concat 
+   toContents (t@Matrix_Interpretation {}) = rmkel "matrixInterpretation" $ concat 
       [ toContents ( domain t )
       , rmkel "dimension"       $ toContents $ dimension t 
       , rmkel "strictDimension" $ toContents $ strictDimension t
       ]
+   toContents (t@Core_Matrix_Interpretation {}) = rmkel "coreMatrixInterpretation" $ concat 
+      [ toContents ( domain t )
+      , rmkel "dimension"       $ toContents $ dimension t
+      , rmkel "indices" $ concatMap toContents $ indices t
+      , toContents (mode t)
+      ]
+
+instance XmlContent Natural where
+  parseContents = error "parsecContents not implemented"
+
+  toContents d =
+    rmkel "nat" $ toContents $ fromIntegral @Natural @Integer d
      
 instance XmlContent Domain where
    parseContents = error "parseContents not implemented"
@@ -333,6 +362,13 @@ instance XmlContent Domain where
        Arctic d -> rmkel "arctic" $ toContents d
        Tropical d -> rmkel  "tropical" $ toContents d
 
+instance XmlContent Mode where
+   parseContents = error "parseContents not implemented"
+
+   toContents d = rmkel "mode" $ case d of
+       E -> rmkel "E" []
+       M -> rmkel "M" []
+
 instance XmlContent Rational where
     parseContents = error "parseContents not implemented"
 
@@ -360,10 +396,14 @@ instance XmlContent Value where
 instance XmlContent Polynomial where
    parseContents = error "parseContents not implemented"
 
-   toContents p = rmkel "polynomial" $ case p of
+   toContents p = id
+                  -- $ rmkel "polynomial"
+                  $ case p of
        Sum     ps -> rmkel "sum"     $ concat ( map toContents ps )
        Product ps -> rmkel "product" $ concat ( map toContents ps )
-       Polynomial_Coefficient c -> rmkel "coefficient" $ toContents c
+       Polynomial_Coefficient c -> id
+         --  $ rmkel "coefficient"
+         $ toContents c
        Polynomial_Variable v -> rmkel "variable" [ nospaceString v ]
 
 instance XmlContent ArithFunction where
@@ -384,8 +424,9 @@ instance XmlContent Coefficient where
    toContents v = case v of
        Matrix vs -> rmkel "matrix" $ concat ( map toContents vs )
        Vector cs -> rmkel "vector" $ concat ( map toContents cs )
-       Coefficient_Coefficient i -> 
-          rmkel "coefficient" $ toContents i
+       Coefficient_Coefficient i -> id
+          -- $ rmkel "coefficient"
+          $ toContents i
 
 instance XmlContent Exotic where
     parseContents = error "parseContents not implemented"
